#!/bin/bash 
# Copyright (c) Members of the EGEE Collaboration. 2004.
# See http://www.eu-egee.org/partners/ for details on the copyright holders.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#-------------------------------------------------------------------------------
# Unit tests for org.glite.data.config-service
# Authors: Paolo Tedesco <paolo.tedesco@cern.ch>
# Version info: $Id: unit-test.sh,v 1.5 2008/06/18 13:55:16 ptedesco Exp $
# Release: $Name:  $
#-------------------------------------------------------------------------------

# check if the WORKSPACE environment variable is defined
if [ -z "${WORKSPACE}" ]; then
    echo "The WORKSPACE environment variable is not set."
    exit 1
fi

test_utils_module="org.glite.data.test-utils"
test_utils_dir="${WORKSPACE}/${test_utils_module}"

# source the test-common script
test_common_script="${test_utils_dir}/test-common.sh"
if [ ! -e ${test_common_script} ]; then
    echo "Checkout or update the ${test_utils_module} module."
    exit 1
fi
. ${test_common_script}

#-------------------------------------------------------------------------------

MODULE_NAME="glite-data-config-service"
MODULE_DIR="${WORKSPACE}/org.glite.data.config-service"
MODULE_TEST_DIR="${GLITE_LOCATION}/share/test/${MODULE_NAME}"
MODULE_CONFIG_DIR="${MODULE_TEST_DIR}/config"

# initialize tests
#-------------------------------------------------------------------------------
function init_unit_test(){
    addpath LD_LIBRARY_PATH ${MODULE_TEST_DIR}/lib
    add_log4cpp_paths
    add_glite_paths
    mkdir -p ${MODULE_TEST_DIR}/{etc,config,bin,var/{log,tmp}} || return 1
}

# run the unit tests
#-------------------------------------------------------------------------------
function config_service_unit_tests(){
    
    (cd ${GLITE_LOCATION}/bin && env \
        GLITE_LOCATION="${MODULE_TEST_DIR}" \
        ${MODULE_TEST_DIR}/bin/glite_data_config_service_test) || {
        return 1
    }
    return 0
    
}

# test the conf-service-test service
#-------------------------------------------------------------------------------
config_service_test__DEPENDS="init_unit_test"
function config_service_test(){
    local SERVICE_NAME="conf-service-test"
    local LOG_PROPERTIES_FILE="${MODULE_TEST_DIR}/etc/${SERVICE_NAME}.log-properties"
    local PROPERTIES_FILE="${MODULE_TEST_DIR}/etc/${SERVICE_NAME}.properties.xml"
    # copy configuration files
    rm -v -f ${LOG_PROPERTIES_FILE} 
    rm -v -f ${PROPERTIES_FILE}
    cp -v ${MODULE_CONFIG_DIR}/${SERVICE_NAME}.* ${MODULE_TEST_DIR}/etc

    # get log file name
    local LOG_FILE=$(grep "log4j\.appender\.file\.fileName" ${MODULE_TEST_DIR}/etc/${SERVICE_NAME}.log-properties | cut -d= -f2)
    if [ "" == "${LOG_FILE}" ]; then
        error "Could not determine log file name."
        return 1
    fi
    if [ -f ${LOG_FILE} ]; then
        debug "Removing previous log file: ${LOG_FILE}"
        rm -v ${LOG_FILE}
    fi

    # Kill Previous errors 
    killall glite_data_config_service
    sleep 1
    killall glite_data_config_service
    sleep 3
    # Start the service
    #---------------------------------------------------------------------------
    info "Starting ${SERVICE_NAME}..."
    (cd ${GLITE_LOCATION}/bin && env \
        GLITE_LOCATION="${MODULE_TEST_DIR}" \
        ${GLITE_LOCATION}/bin/glite_data_config_service -v ${SERVICE_NAME} start) || {
        error "${SERVICE_NAME} failed to start. Check log file ${LOG_FILE}"   
        return 1
    }
    # Check the service status
    #---------------------------------------------------------------------------
    info "Checking ${SERVICE_NAME} status..."
    (cd ${GLITE_LOCATION}/bin && env \
        GLITE_LOCATION="${MODULE_TEST_DIR}" \
        ${GLITE_LOCATION}/bin/glite_data_config_service -v ${SERVICE_NAME} status) || {
        error "Failed checking status of ${SERVICE_NAME}. Check log file ${LOG_FILE}"   
        return 1
    }

    # Check that the log file contains the expected entries
    #---------------------------------------------------------------------------
    info "Waiting..."
    sleep 5
    info "Checking initialization and configuration parameters after service start..."
    local retval=0

    # initialization values
    local INIT_PARAMS_FILE=$(mktemp -p /tmp test-glite-data-config-service-XXXXXXXXXX)
    local INIT_PARAM_NAME="TestInitParam"
    local INIT_PARAM_EXPECTED="A Value"
    grep -o "Init Parameter \w\+ value is .*" ${LOG_FILE} | sed 's/Init Parameter \(\w\+\) value is \(.*\)/local \1="\2"/' > ${INIT_PARAMS_FILE}
    #local INIT_PARAM_VALUE=$(grep "^${INIT_PARAM_NAME}=" ${INIT_PARAMS_FILE} | cut -d= -f2)
    # source the file and get the value of the parameter
    . ${INIT_PARAMS_FILE}
    local INIT_PARAM_VALUE=${!INIT_PARAM_NAME}
    if [ "" == "${INIT_PARAM_VALUE}" ]; then
        error "Initialization parameter ${INIT_PARAM_NAME} not found in log file"
        retval=1
    else
        if [ "${INIT_PARAM_EXPECTED}" == "${INIT_PARAM_VALUE}" ]; then
            info "Value of \"${INIT_PARAM_NAME}\" is \"${INIT_PARAM_VALUE}\""
        else
            error "Value of \"${INIT_PARAM_NAME}\" is \"${INIT_PARAM_VALUE}\" (expected \"${INIT_PARAM_EXPECTED}\")"
            retval=1
        fi
    fi

    # configuration values
    local CONFIG_PARAMS_FILE=$(mktemp -p /tmp test-glite-data-config-service-XXXXXXXXXX)
    local CONFIG_PARAM_NAME="interval"
    local CONFIG_PARAM_EXPECTED="5"
    grep -o "Config Parameter \w\+ value is .*" ${LOG_FILE} | sed 's/Config Parameter \(\w\+\) value is \(.*\)/\1=\2/' > ${CONFIG_PARAMS_FILE}
    #local CONFIG_PARAM_VALUE=$(grep "^${CONFIG_PARAM_NAME}=" ${CONFIG_PARAMS_FILE} | cut -d= -f2)
    . ${CONFIG_PARAMS_FILE}
    local CONFIG_PARAM_VALUE=${!CONFIG_PARAM_NAME}
    if [ "" == "${CONFIG_PARAM_VALUE}" ]; then
        error "Configuration parameter ${CONFIG_PARAM_NAME} not found in log file"
        retval=1
    else
        if [ "${CONFIG_PARAM_EXPECTED}" == "${CONFIG_PARAM_VALUE}" ]; then
            info "Value of \"${CONFIG_PARAM_NAME}\" is \"${CONFIG_PARAM_VALUE}\""
        else
            error "Value of \"${CONFIG_PARAM_NAME}\" is \"${CONFIG_PARAM_VALUE}\" (expected \"${CONFIG_PARAM_EXPECTED}\")"
            retval=1
        fi
    fi

    # Reconfigure the service
    #---------------------------------------------------------------------------
    info "Reconfiguring ${SERVICE_NAME}..."
    debug "Modifying configuration file"
    XSLT_TRANSFORM=$(mktemp -p /tmp test-glite-data-config-service-reconfigure-XXXXXXXXXX)
    # create the xslt transform file
    cat > ${XSLT_TRANSFORM} << EOD
<?xml version="1.0" encoding="UTF-8" ?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
    <xsl:output method="xml" encoding="UTF-8" indent="yes"/>
    <xsl:strip-space elements="*"/>
    <!-- identity transform -->
    <xsl:template match="@*|node()">
        <xsl:copy>
            <xsl:apply-templates select="@*|node()" />
        </xsl:copy>
    </xsl:template>
    <!-- replace the transfer service node -->
    <xsl:template match="/service/components/component/config/param">
        <xsl:choose>
            <xsl:when test="@name='interval'">
                <param name="interval">
                    <value>3</value>
                </param>
            </xsl:when>
            <xsl:otherwise>
                <xsl:copy>
                    <xsl:apply-templates select="@*|node()" />
                </xsl:copy>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
</xsl:stylesheet>
EOD

    # apply transformation
    cp ${PROPERTIES_FILE} ${PROPERTIES_FILE}.backup
    xsltproc -o ${PROPERTIES_FILE} ${XSLT_TRANSFORM} ${PROPERTIES_FILE}.backup

    # actual reconfiguration
    debug "Service reconfig"
    (cd ${GLITE_LOCATION}/bin && env \
        GLITE_LOCATION="${MODULE_TEST_DIR}" \
        ${GLITE_LOCATION}/bin/glite_data_config_service -v ${SERVICE_NAME} reconfig) || {
        error "Failed reconfiguring ${SERVICE_NAME}. Check log file ${LOG_FILE}"   
        retval=1
    }
    
    # check new configuration parameters
    info "Waiting..."
    sleep 5
    info "Checking values of configuration parameters after reconfiguration"
    CONFIG_PARAM_NAME="interval"
    CONFIG_PARAM_EXPECTED="3"
    grep -o "Config Parameter \w\+ value is .*" ${LOG_FILE} | sed 's/Config Parameter \(\w\+\) value is \(.*\)/\1=\2/' > ${CONFIG_PARAMS_FILE}
    . ${CONFIG_PARAMS_FILE}
    local CONFIG_PARAM_VALUE=${!CONFIG_PARAM_NAME}
    if [ "" == "${CONFIG_PARAM_VALUE}" ]; then
        error "Configuration parameter ${CONFIG_PARAM_NAME} not found in log file"
        retval=1
    else
        if [ "${CONFIG_PARAM_EXPECTED}" == "${CONFIG_PARAM_VALUE}" ]; then
            info "Value of \"${CONFIG_PARAM_NAME}\" is \"${CONFIG_PARAM_VALUE}\""
        else
            error "Value of \"${CONFIG_PARAM_NAME}\" is \"${CONFIG_PARAM_VALUE}\" (expected \"${CONFIG_PARAM_EXPECTED}\")"
            retval=1
        fi
    fi

    # Stop the service
    #---------------------------------------------------------------------------
    info "Stopping ${SERVICE_NAME}..."
    (cd ${GLITE_LOCATION}/bin && env \
        GLITE_LOCATION="${MODULE_TEST_DIR}" \
        ${GLITE_LOCATION}/bin/glite_data_config_service -v ${SERVICE_NAME} stop) || {
        error "${SERVICE_NAME} failed to stop. Check log file ${LOG_FILE}"   
        return 1
    }

    # cleanup
    rm ${INIT_PARAMS_FILE}
    rm ${CONFIG_PARAMS_FILE}
    rm ${XSLT_TRANSFORM}
    
    return ${retval}
}

# config_service_test_static
#-------------------------------------------------------------------------------
config_service_test_static__DEPENDS="init_unit_test"
function config_service_test_static(){
    local EXE_NAME="test-config-service-static"
    local LOG_PROPERTIES_FILE="${MODULE_TEST_DIR}/etc/${EXE_NAME}.log-properties"
    local PROPERTIES_FILE="${MODULE_TEST_DIR}/etc/${EXE_NAME}.properties.xml"
    # copy configuration files
    rm -v -f ${LOG_PROPERTIES_FILE} 
    rm -v -f ${PROPERTIES_FILE}
    cp -v ${MODULE_CONFIG_DIR}/${EXE_NAME}.* ${MODULE_TEST_DIR}/etc
    (cd ${GLITE_LOCATION}/bin && env \
        GLITE_LOCATION="${MODULE_TEST_DIR}" \
        ${MODULE_TEST_DIR}/bin/glite_data_config_service_test_static) || {
        error "Could not start glite_data_config_service_test_static"
        return 1
    }
}

# config_service_test_hang
#-------------------------------------------------------------------------------
config_service_test_hang__DEPENDS="init_unit_test"
function config_service_test_hang(){
    local SERVICE_NAME="conf-service-hang-test"
    local LOG_PROPERTIES_FILE="${MODULE_TEST_DIR}/etc/${SERVICE_NAME}.log-properties"
    local PROPERTIES_FILE="${MODULE_TEST_DIR}/etc/${SERVICE_NAME}.properties.xml"
    # copy configuration files
    rm -v -f ${LOG_PROPERTIES_FILE} 
    rm -v -f ${PROPERTIES_FILE}
    cp -v ${MODULE_CONFIG_DIR}/${SERVICE_NAME}.* ${MODULE_TEST_DIR}/etc
    # Kill Previous errors 
    killall glite_data_config_service
    sleep 1
    killall glite_data_config_service
    sleep 3
    # Start the service
    #---------------------------------------------------------------------------
    info "Starting ${SERVICE_NAME}..."
    (cd ${GLITE_LOCATION}/bin && env \
        GLITE_LOCATION="${MODULE_TEST_DIR}" \
        ${GLITE_LOCATION}/bin/glite_data_config_service -v ${SERVICE_NAME} start) || {
        error "${SERVICE_NAME} failed to start. Check log file ${LOG_FILE}"   
        return 1
    }
    # Check the service status
    #---------------------------------------------------------------------------
    info "Checking ${SERVICE_NAME} status..."
    (cd ${GLITE_LOCATION}/bin && env \
        GLITE_LOCATION="${MODULE_TEST_DIR}" \
        ${GLITE_LOCATION}/bin/glite_data_config_service -v ${SERVICE_NAME} status) || {
        error "Failed checking status of ${SERVICE_NAME}. Check log file ${LOG_FILE}"   
        return 1
    }
    # Stop the service (note that this must fail)
    #---------------------------------------------------------------------------
    info "Stopping ${SERVICE_NAME} (this operation must fail)"
    (cd ${GLITE_LOCATION}/bin && env \
        GLITE_LOCATION="${MODULE_TEST_DIR}" \
        ${GLITE_LOCATION}/bin/glite_data_config_service -v ${SERVICE_NAME} stop) && {
        error "${SERVICE_NAME} stopped (was expected to hang). Check log file ${LOG_FILE}"   
        return 1
    }
    # kill the service
    #---------------------------------------------------------------------------
    info "Killing ${SERVICE_NAME}..."
    (cd ${GLITE_LOCATION}/bin && env \
        GLITE_LOCATION="${MODULE_TEST_DIR}" \
        ${GLITE_LOCATION}/bin/glite_data_config_service -v ${SERVICE_NAME} kill) || {
        error "${SERVICE_NAME} could not be killed. Check log file ${LOG_FILE}"   
        return 1
    }
}

# test_all
#-------------------------------------------------------------------------------
test_all__DEPENDS="config_service_unit_tests config_service_test config_service_test_static config_service_test_hang"
function test_all(){
    true
}

process_command_line ${*}


